/*
  -------------------- file bitpar.c
  Created 18 May 94 by Andrew Shapira, Rensselaer Polytechnic Institute.

	Copyright (C) 1994 Andrew Shapira.

	This file is part of the Ntuplib library.

	Ntuplib is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	Ntuplib is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with the Ntuplib Library; if not, write to the Free Software
	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/


/*  USERS MUST CALL bitword_global_init() BEFORE CALLING THESE FUNCTIONS.
*/

#define SRCFILE_LIBPROG_BITPAR

#include "our_common.h"
#include "bitpar.h"

/* Calculate how many levels are needed to hold a given positive integer v.
   This is ceil(lg v).
   MACHINE DEPENDENT.
   TODO - move to bits.c
*/
int
bitpar_levels_needed(
const BITWORD v
)
{
  const BITWORD t = (BITWORD) 1;
  if (v < (t << 1))   return(1);
  if (v < (t << 2))   return(2);
  if (v < (t << 3))   return(3);
  if (v < (t << 4))   return(4);
  if (v < (t << 5))   return(5);
  if (v < (t << 6))   return(6);
  if (v < (t << 7))   return(7);
  if (v < (t << 8))   return(8);
  if (v < (t << 9))   return(9);
  if (v < (t << 10))  return(10);
  if (v < (t << 11))  return(11);
  if (v < (t << 12))  return(12);
  if (v < (t << 13))  return(13);
  if (v < (t << 14))  return(14);
  if (v < (t << 15))  return(15);
  if (v < (t << 16))  return(16);
  if (v < (t << 17))  return(17);
  if (v < (t << 18))  return(18);
  if (v < (t << 19))  return(19);
  if (v < (t << 20))  return(20);
  if (v < (t << 21))  return(21);
  if (v < (t << 22))  return(22);
  if (v < (t << 23))  return(23);
  if (v < (t << 24))  return(24);
  if (v < (t << 25))  return(25);
  if (v < (t << 26))  return(26);
  if (v < (t << 27))  return(27);
  if (v < (t << 28))  return(28);
  if (v < (t << 29))  return(29);
  if (v < (t << 30))  return(30);
  if (v < (t << 31))  return(31);
  return(32);
}
#ifdef COULDDO    /* not tested, is almost always overkill */
static int
bit_need(
const unsigned long i
)
{
  if (i < (1 << 16)) {
    if (i < (1 << 8)) {       /* between 1 and 8 */
      if (i < (1 << 4)) {
        if (i < (1 << 2)) {
          if (i < (1 << 1))     return(1);
          else                  return(2);
        } else {
          if (i < (1 << 3))     return(3);
          else                  return(4);
        }
      } else {
        if (i < (1 << 6)) {
          if (i < (1 << 5))     return(5);
          else                  return(6);
        } else {
          if (i < (1 << 7))     return(7);
          else                  return(8);
        }
      }
    }
    else {                      /* between 9 and 16 */
      if (i < (1 << 12)) {
        if (i < (1 << 10)) {
          if (i < (1 << 9))     return(9);
          else                  return(10);
        } else {
          if (i < (1 << 11))    return(11);
          else                  return(12);
        }
      } else {
        if (i < (1 << 14)) {
          if (i < (1 << 13))    return(13);
          else                  return(14);
        } else {
          if (i < (1 << 15))    return(15);
          else                  return(16);
        }
      }
    }
  } else {
    if (i < (1 << 24)) {        /* between 17 and 24 */
      if (i < (1 << 20)) {
        if (i < (1 << 18)) {
          if (i < (1 << 17))    return(17);
          else                  return(18);
        } else {
          if (i < (1 << 19))    return(19);
          else                  return(20);
        }
      } else {
        if (i < (1 << 22)) {
          if (i < (1 << 21))    return(21);
          else                  return(22);
        } else {
          if (i < (1 << 23))    return(23);
          else                  return(24);
        }
      }
    } else {                  /* between 25 and 32 */
      if (i < (1 << 28)) {
        if (i < (1 << 26)) {
          if (i < (1 << 25))    return(25);
          else                  return(26);
        } else {
          if (i < (1 << 27))    return(27);
          else                  return(28);
        }
      } else {
        if (i < (1 << 30)) {
          if (i < (1 << 29))    return(29);
          else                  return(30);
        } else {
          if (i < (1 << 31))    return(31);
          else                  return(32);
        }
      }
    }
  }
}
#endif /*COULDDO*/
