/*
  -------------------- file common.h
  Created 1992 by Andrew Shapira, Rensselaer Polytechnic Institute.

	Copyright (C) 1992 Andrew Shapira.

	This file is part of the Ntuplib library.

	Ntuplib is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	Ntuplib is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with the Ntuplib Library; if not, write to the Free Software
	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

/*
  ----------------------------------------------------------------------
  ----------------------------------------------------------------------
  ----------------------------------------------------------------------
  ---------- YOU USUALLY SHOULD NOT INCLUDE THIS FILE DIRECTLY. --------
  ----------------------------------------------------------------------
  ----------------------------------------------------------------------
  ----------------------------------------------------------------------

  Instead, make a file in whatever source directory you have (say,
  our_common.h, the reccommended name) and include our_common.h from
  your C files.  Inside our_common.h you should define:
    COMMON_INCLUDE
      You need to define this before including this file; this should help
      keep you from including this file by mistake from the wrong place.

  You should also define or comment out the definitions for:
    ASSERTIONS_ON
    HEAVY_ASSERTIONS_ON
    GCCHAPPY
      Add code that supresses certain warnings from gcc.

  You should define (or not define) the following symbols, in the source
  file that includes our_common.h:
    DEBUG
  
  This will make it easier for you to change these flags without having to
  modify many programs.  This directory contains a prototypical our_common.h
  file.  Often, you will find other things to put in our_common.h besides
  things related to common.h.

  ----------------------------------------------------------------------

  See vector.h for an example of how to do inline functions.

  You can depend on this file (common.h) to include stdio.h.

  A note about "const" in variable declarations: a lack of a "const" does
  not indicate an intent that the parameter not be used the way a "const"
  would be used (that is, as a read-only entity).  One reason for this is
  the problem with two- and higher level nesting of pointers and array
  references:

      const char  **p;
      char        **q;

  p = q;   const is two levels deep; not legal according to ANSI standard
  *p = *q; const is one level deep; is legal
*/

#ifndef COMMON_INCLUDE
#error common_include problem
#endif  /* COMMON_INCLUDE */

#ifndef _020992_121104_common
#define _020992_121104_common

#ifndef FILE
#include <stdio.h>
#endif  /*FILE*/

#define __SUN__

#ifndef P
#if     defined(__STDC__) || defined(__GNUC__)
#define P(xx) xx
#ifndef __stdlib_h
#include <stdlib.h>
#endif  /* __stdlib_h */
#else
#define P(xx) ()
#define const
#define volatile
#ifndef __sys_types_h
#include <sys/types.h>
#endif  /* __sys_types_h */
#endif
#endif /* P */

#ifndef __GNUC__
#define __inline__
#endif

#ifdef  DEBUG
#define D(xx)   do { xx } while (0) ;
#else   /* DEBUG */
#define D(xx)
#endif  /* DEBUG */

#ifdef GCCHAPPY
#define DO_GCCHAPPY(xx)   do { xx } while (0) ;
#else
#define DO_GCCHAPPY(xx)
#endif

extern void fatal P((char const * const));

#ifdef  ASSERTIONS_ON
#ifdef  __STDC__
/* xx must not contain quotes, or have side effects */
#define ASSERT(xx)  do {if (! (xx)) fatal("Failed assertion:" \
      "   File: " __FILE__ "\nTest:  " #xx ); } while (0)
#else /* __STDC__ */
#define ASSERT(xx)  do {if (! (xx)) fatal("Failed assertion:"); } while (0)
#endif  /* __STDC__ */
#else /* ASSERTIONS_ON */
#define ASSERT(xx)
#endif  /* ASSERTIONS_ON */

/*  HEAVY_ASSERTIONS_ON is intended to allow you to have two levels of
    assertions to segregate your assertions into ones that are fast and
    ones that are not.
*/
#ifdef  HEAVY_ASSERTIONS_ON
#ifdef  __STDC__
/* xx must not contain quotes, or have side effects */
#define HEAVY_ASSERT(xx)  do {if (! (xx)) fatal("Failed heavy assertion:" \
      "   File: " __FILE__ "\nTest:  " #xx ); } while (0)
#else /* __STDC__ */
#define HEAVY_ASSERT(xx)  do {if (! (xx)) fatal("Failed heavy assertion:"); } while (0)
#endif  /* __STDC__ */
#else /* HEAVY_ASSERTIONS_ON */
#define HEAVY_ASSERT(xx)
#endif  /* HEAVY_ASSERTIONS_ON */


#define MAX(xx,yy)    ( ((xx) > (yy)) ? (xx) : (yy) )
#define MIN(xx,yy)    ( ((xx) < (yy)) ? (xx) : (yy) )
#define ABS(xx)       ( ((xx) >= 0) ? (xx) : (-1 * (xx)) )




/* ----- memory manipulation */

void *memcpy P((void *, const void *, size_t));

#ifdef  __SUN__
/* bcopy(src, dest, len) */
void bcopy P((const void *, void *, size_t));
/* memmove(dest, src, len) */
#define memmove(xx,yy,zz) bcopy((yy),(xx),(zz))
/* random numbers from random() are in range 0 .. MAXRANDOM */
#define MAXRANDOM 2147483646   /* might not be tight, but it's big */
#else
void *memmove P((void *, const void *, size_t));
#endif  /*__SUN__*/

int   memcmp P((const void *, const void *, size_t));
void *memchr P((const void *, int, size_t));
void *memset P((void *, int, size_t));

#endif /* _020992_121104_common */
