/*
  -------------------- file getnum.c

	Copyright (C) 1992 Andrew Shapira.

	This file is part of the Ntuplib library.

	Ntuplib is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	Ntuplib is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with the Ntuplib Library; if not, write to the Free Software
	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include <stdio.h>
#include <ctype.h>
#include "our_common.h"
#include "getnum.h"


/*
  Read a file from FILE pointer f.
      White space is ignored.
      Comments are ignored.
      A comment is a '%' or 'c' and all following text on the same line.
  An unrecognized character ends the current number; if it begins a
  number, this is considered a failure.
    
  Returns 1 if an integer was successfully read, else returns 0.
  If successful, the integer is returned in *retint.

  If the number is longer than our buffer size, we return 0.
*/
int
getnum(
FILE  *f,
int   *retint         /* where to return integer read in */
)
{
  char    buff[256];
  int     c;                /* latest character read */
  int     in_comment;
  int     len;              /* number of chars in number */

  int getc P(( FILE * ));
  int ungetc P(( int, FILE * ));

  /* skip white space and comments */
  in_comment = 0;
  while (1) {
    c = getc(f);
    if (c == EOF)  return(0);
    if (in_comment) {
      if (c == '\n')  in_comment = 0;
      continue;
    }
    if (isspace(c))  continue;
    if ((c == '%') || (c == 'c'))  in_comment = 1;
    else  break;
  }

  /* at this point c!=EOF and c is the first character of the number */
  len = 0;
  if (c == '-') {
    buff[len++] = c;
    c = getc(f);
    if (c == EOF)  return(0);
  }
  /* at this point c!=EOF and c is the first character of the number */
  while (1) {
    if ((c < '0') || (c > '9'))  {
      if (ungetc(c, f) == EOF)  fatal("getnum: ungetc");
      break;
    }
    if (len >= sizeof(buff)-1)  return(0);    /* number is too big */
    buff[len++] = c;
    c = getc(f);
    if (c == EOF)  break;
  }
  if ((len == 0) || ((len == 1) && (buff[0] == '-')))
    return(0);
  buff[len++] = '\0';
  *retint = atoi(buff);
  return(1);
}
