/*
  -------------------- file matrix.c

	Copyright (C) 1992 Andrew Shapira.

	This file is part of the Ntuplib library.

	Ntuplib is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	Ntuplib is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with the Ntuplib Library; if not, write to the Free Software
	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/


#ifndef _020992_124132_matrix
#define _020992_124132_matrix

#include "common.h"
#include "memalloc.h"

/*  The naming convention for matrix routines: without a specifier, the
    matrix type is int.  This is the same convention that is used for vectors.

    A note about consts: when we would like to do something like
    const int * const * const m, we are foiled because we will get
    typechecking mismatches.  ANSI C only does const checking one
    level deep.  So when you see "int * const * const m", this may
    mean that we would also like the outermost level to have a const.
    The policy is to put as many consts as possible.  (Some functions
    were written before I started following this policy.)
*/

void mat_row_histogram P((int * const * const , const int, int * const * const ));
void mat_col_histogram P((int * const * const , const int, int * const * const ));
void mat_uchar_histogram
  P(( unsigned char **, const int, const int, const int, const int, int * ));
void mat_print2 P(( FILE *, int * const * const, const int, const int));
void mat_uchar_print2 P(( FILE *, unsigned char * const * const,
  const int, const int ));
void mat_double_print2 P(( FILE *, double * const * const,
  const int, const int, const int ));
void mat_print P(( FILE *, int * const * const, const int));
void mat_uchar_print P(( FILE *, unsigned char * const * const, const int));
int mat_print_listformat P(( FILE *, int * const * const,
  const int, const int, int * ));
int mat_uchar_read2 P((FILE *, unsigned char **, const int, const int));
int mat_uchar_read2_with_dim
  P(( FILE *, unsigned char ***, int *, int *, void*(*) P((const int)) ));
int mat_read2 P((FILE *, int **, const int, const int ));
int mat_uchar_read P((FILE *, unsigned char **, const int ));
int mat_read P((FILE *, int **, const int ));
int mat_uchar_is_main_diag0 P((unsigned char **, const int));
int mat_is_main_diag0 P((int **, const int));
int mat_uchar_is_main_diag1 P((unsigned char **, const int));
int mat_is_main_diag1 P((int **, const int));
void mat_table_to_mat
    P(( int **, const int, const int, const int, const int, int ** ));
void mat_scale2 P(( int **, const int, const int, const int, const int ));
void mat_uchar_scale2
  P(( unsigned char **, const int, const int, const int, const int ));
void mat_double_scale2
  P(( double **, const int, const int, const double, const double ));
void mat_uchar_permute P((unsigned char **,unsigned char **,const int,int *));
void mat_uchar_permute_random P(( unsigned char **, const int ));
void mat_uchar_random_binary2 P(( unsigned char **,
  const int, const int, const unsigned long, const unsigned long ));
void mat_uchar_random_binary
  P(( unsigned char **, const int, const unsigned long, const unsigned long ));
void mat_uchar_trim2 P(( unsigned char **, const int, const int, const int,
  unsigned char ***, int *, int *, void *(*) P((const int)) ));
void mat_uchar_neighbors P(( unsigned char * const * const,
                                const int, int * const *const neighbors ));
int ** mat_uchar_alloc_neighbors P(( unsigned char * const * const,
                                        const int, void *(*) P((const int)) ));

/**************************************************************************
  inline functions (be careful about changing the order of func. definition)
**************************************************************************/

/* declarations (included for compatibility with non-gnuc compilers) */

#ifndef __GNUC__
extern int ** mat_alloc2 P(( const int, const int, void *(*) P((const int)) ));
unsigned char ** mat_uchar_alloc2
                 P(( const int, const int, void *(*) P((const int)) ));
extern double ** mat_double_alloc2
                 P(( const int, const int, void *(*) P((const int)) ));
extern int ** mat_alloc P(( const int, void *(*) P((const int)) ));
extern unsigned char ** mat_uchar_alloc
                 P(( const int, void *(*) P((const int)) ));
extern double ** mat_double_alloc P(( const int, void *(*) P((const int)) ));
extern void mat_free P(( int **, void (*) P((void *)) ));
extern void mat_uchar_free P(( unsigned char **, void (*) P((void *)) ));
extern void mat_double_free P(( double **, void (*) P((void *)) ));
extern void mat_copy2 P((int **, const int, const int, int ** ));
extern void mat_uchar_copy2
        P((unsigned char **, const int, const int, unsigned char ** ));
extern void mat_uchar_copy_to_double2
        P((unsigned char **, const int, const int, double ** ));
extern void mat_double_copy_to_uchar2
        P((double **, const int, const int, unsigned char ** ));
extern void mat_copy P((int **, int **, const int ));
extern void mat_uchar_copy
                    P((unsigned char **, unsigned char **, const int n));
extern void mat_copy_to_uchar2
    P(( int **, const int, const int, unsigned char ** ));
extern void mat_copy_var_rowlength P((int **, int **, const int, const int *));
extern void mat_uchar_zero2 P((unsigned char **, const int, const int ));
extern void mat_uchar_zero P((unsigned char **, const int ));
extern void mat_zero2 P((int * const * const , const int, const int ));
extern void mat_zero P((int * const * const , const int ));
extern void mat_set2 P(( int **, const int, const int, const int ));
extern void mat_uchar_set2
  P (( unsigned char **, const int, const int, const int ));
extern void mat_double_set2
  P(( double **, const int, const int, const double ));
extern void mat_set P(( int **, const int, const int ));
extern void mat_uchar_set P(( unsigned char **, const int, const int ));
extern void mat_uchar_zero_lowright P((unsigned char * const * const,
  const int, const int));
extern void mat_add_scalar P((int **, const int, const int ));
extern void mat_add_scalar2 P((int **, const int, const int, const int ));
extern void mat_uchar_add_scalar P((unsigned char **, const int, const int ));
extern void mat_uchar_add_scalar2
  P((unsigned char **, const int, const int, const int ));
extern int mat_sum2 P(( int **, const int, const int ));
extern int mat_positive_sum2 P(( int **, const int, const int ));
extern void mat_uchar_row_sum2
  P(( unsigned char * const * const, const int, const int, int * const ));
extern void mat_uchar_row_sum
  P(( unsigned char * const * const, const int, int * const ));
extern void mat_row_sum2
  P(( int * const * const, const int, const int, int * const ));
extern void mat_row_sum P(( int * const * const, const int, int * const ));
extern int mat_uchar_sum2 P(( unsigned char **, const int, const int ));
extern double mat_double_sum2 P(( double **, const int, const int ));
extern int mat_positive_sum P (( int **, const int ));
extern int mat_uchar_sum P (( unsigned char **, const int ));
extern int mat_sum_squares2 P(( int **, const int, const int ));
extern int mat_uchar_sum_squares2 P(( unsigned char **, const int, const int ));
extern int mat_sum_squares P(( int **, const int ));
extern int mat_uchar_sum_squares P(( unsigned char **, const int ));
extern void mat_transpose P(( int **, int **l, const int ));
extern void mat_uchar_minmax2 P((unsigned char * const * const, const int,
  const int, int * const, int * const));
extern void mat_double_minmax2 P((double * const * const ,
  const int, const int, double * const, double * const));
extern void mat_minmax2 P((int * const *, const int, const int,
  int * const, int * const));
extern void mat_minmax P((int * const *, const int,
  int * const, int * const));
extern int mat_uchar_cmp2 P((unsigned char **, unsigned char **, const int, const int));
extern int mat_cmp2 P((int **, int **, const int, const int));
extern int mat_uchar_cmp P((unsigned char **, unsigned char **, const int));
extern int mat_cmp P((int **, int **, const int));
extern int mat_is0 P((int **, const int));
extern int mat_uchar_is_symmetric2 P((unsigned char **, const int, const int ));
extern int mat_is_symmetric2 P((int **, const int, const int ));
extern int mat_uchar_is_symmetric P((unsigned char **, const int));
extern int mat_is_symmetric P((int **, const int));
extern void mat_zero_3d P((int ** const * const, const int,
  const int, const int));
extern void mat_apply_map P(( int **, const int, const int,
      const int, const int *, int ** ));
extern void mat_uchar_apply_map P(( unsigned char **, const int,
      const int, const int, const int *, unsigned char ** ));
#endif  /*__GNUC__*/

/*  note: inline is defined to the empty string unless we're using gnuc */
#undef  INLINE
#ifdef  SRCFILE_LIBPROG_MATRIX
#define INLINE  __inline__
#else
#define INLINE  __inline__ extern
#endif

#if defined(__GNUC__) || defined(SRCFILE_LIBPROG_MATRIX)

/*  Allocate an int matrix with specified number of rows and columns.
    Uses only one call to memory allocator (fast, is convenient for freeing)

    Note: this function and its clones sometimes waste a small amount of
    memory in order to have increased portability, in conjunction with
    only needing one call to the memory allocator.  The waste comes from
    the use of MEM_ALIGN_REQUEST.  The amount wasted depends on the
    machine, and also the allocation request.  A typical upper bound
    on the amount wasted is 11 bytes, with the usual waste being less
    than that.  This is probably of concern only when the caller is
    doing many allocations of small matrices.
*/
INLINE int **
mat_alloc2(
const int rows,
const int cols,
void  *(*allocfunc) P((const int))
)
{
  int **m;
  int i;
  unsigned char *rowp;
  unsigned char *buff;
  const int buff1_size = MEM_ALIGN_REQUEST(rows*sizeof(int *));
  const int buff2_size = MEM_ALIGN_REQUEST(rows*cols*sizeof(int));

  buff = (*allocfunc) (buff1_size + buff2_size);
  m = (int **) buff;
  rowp = & buff[buff1_size];
  for (i = 0; i < rows; i++) {
    m[i] = (int *) rowp;
    rowp += cols*sizeof(int);
  }
  return(m);
}

INLINE unsigned char **
mat_uchar_alloc2(
const int rows,
const int cols,
void  *(*allocfunc) P((const int))
)
{
  unsigned char **m;
  int i;
  unsigned char *rowp;
  unsigned char *buff;
  const int buff1_size = MEM_ALIGN_REQUEST(rows*sizeof(unsigned char *));
  const int buff2_size = MEM_ALIGN_REQUEST(rows*cols*sizeof(unsigned char));

  buff = (*allocfunc) (buff1_size + buff2_size);
  m = (unsigned char **) buff;
  rowp = & buff[buff1_size];
  for (i = 0; i < rows; i++) {
    m[i] = (unsigned char *) rowp;
    rowp += cols*sizeof(unsigned char);
  }
  return(m);
}

INLINE double **
mat_double_alloc2(
const int rows,
const int cols,
void  *(*allocfunc) P((const int))
)
{
  double  **m;
  int i;
  unsigned char *rowp;
  unsigned char *buff;
  const int buff1_size = MEM_ALIGN_REQUEST(rows*sizeof(double *));
  const int buff2_size = MEM_ALIGN_REQUEST(rows*cols*sizeof(double));

  buff = (*allocfunc) (buff1_size + buff2_size);
  m = (double **) buff;
  rowp = & buff[buff1_size];
  for (i = 0; i < rows; i++) {
    m[i] = (double *) rowp;
    rowp += cols*sizeof(double);
  }
  return(m);
}


/*  Allocate an n by n int matrix. */
INLINE int **
mat_alloc(
const int n,
void  *(*allocfunc) P((const int))
)
{
  return(mat_alloc2(n, n, allocfunc));
}

INLINE unsigned char **
mat_uchar_alloc(
const int n,
void  *(*allocfunc) P((const int))
)
{
  return(mat_uchar_alloc2(n, n, allocfunc));
}

INLINE double **
mat_double_alloc(
const int n,
void  *(*allocfunc) P((const int))
)
{
  return(mat_double_alloc2(n, n, allocfunc));
}

/*  Free a matrix that was allocated with mat_alloc().  No knowledge is
    needed of the matrix's size.  It is allowable that the row vector
    addresses be shifted around since when they were allocated.
*/
INLINE void
mat_free(
int   **m,
void  (*freefunc) P((void *))
)
{
  (*freefunc) (m);
}

INLINE void
mat_uchar_free(
unsigned char   **m,
void  (*freefunc) P((void *))
)
{
  (*freefunc) (m);
}

INLINE void
mat_double_free(
double    **m,
void      (*freefunc) P((void *))
)
{
  (*freefunc) (m);
}

/*  copy a matrix to another matrix of the same size
*/
INLINE void
mat_copy2(
int **msource,
const int       rows,
const int       cols,
int **mdest
)
{
  int   row,col;

  for (row = 0; row < rows; row++) {
    for (col = 0; col < cols; col++) {
      mdest[row][col] = msource[row][col];
    }
  }
}

INLINE void
mat_uchar_copy2(
unsigned char **msource,
const int       rows,
const int       cols,
unsigned char **mdest
)
{
  int   row,col;

  for (row = 0; row < rows; row++) {
    for (col = 0; col < cols; col++) {
      mdest[row][col] = msource[row][col];
    }
  }
}

INLINE void
mat_uchar_copy_to_double2(
unsigned char   **msource,
const int       rows,
const int       cols,
double          **mdest
)
{
  int   row,col;

  for (row = 0; row < rows; row++) {
    for (col = 0; col < cols; col++) {
      mdest[row][col] = (double) msource[row][col];
    }
  }
}

INLINE void
mat_double_copy_to_uchar2(
double          **msource,
const int       rows,
const int       cols,
unsigned char   **mdest
)
{
  double  v;
  int   row,col;

  for (row = 0; row < rows; row++) {
    for (col = 0; col < cols; col++) {
      v = msource[row][col];
      ASSERT((v >= 0.0) && (v < 256.0));
      mdest[row][col] = (unsigned char) v;
    }
  }
}

INLINE void
mat_copy(
int **mdest,
int **msource,
const int           n                  /* matrix is n*n */
)
{
  mat_copy2(msource,n,n,mdest);
}

INLINE void
mat_uchar_copy(
unsigned char **mdest,
unsigned char **msource,
const int           n                  /* matrix is n*n */
)
{
  mat_uchar_copy2(msource,n,n,mdest);
}

INLINE void
mat_copy_to_uchar2(
int **msource,
const int   rows,
const int   cols,
unsigned char **mdest
)
{
  int row,col;

  for (row = 0; row < rows; row++) {
    for (col = 0; col < cols; col++) {
      mdest[row][col] = msource[row][col];
    }
  }
}

/*  Copy a matrix whose rows are of variable length.
    The length of row i is given by row_length[i].
*/
INLINE void
mat_copy_var_rowlength(
int   **dest,
int   **source,     /*  const int **  */
const int   rows,   /* number of rows */
const int   *row_length
)
{
  int   row, col,l;

  for (row = 0; row < rows; row++) {
    l = row_length[row];
    for (col = 0; col < l; col++)
      dest[row][col] = source[row][col];
  }
}

INLINE void
mat_uchar_zero2(
unsigned char * const * const m,
const int     rows,
const int     cols
)
{
  int   row, col;

  for (row = 0; row < rows; row++) {
    for (col = 0; col < cols; col++)
      m[row][col] = 0;
  }
}

/* zero all the elements of a matrix */
INLINE void
mat_uchar_zero(
unsigned char * const * const m,
const int     n                  /* copy upper left n*n components */
)
{
  int   row, col;

  for (row = 0; row < n; row++) {
    for (col = 0; col < n; col++)
      m[row][col] = 0;
  }
}

INLINE void
mat_zero2(
int * const * const m,
const int     rows,
const int     cols
)
{
  int   row, col;

  for (row = 0; row < rows; row++) {
    for (col = 0; col < cols; col++)
      m[row][col] = 0;
  }
}

INLINE void
mat_zero(
int * const * const m,
const int     n
)
{
  mat_zero2(m,n,n);
}

INLINE void
mat_set2(
int **m,
const int     rows,
const int     cols,
const int     v
)
{
  int   row, col;

  for (row = 0; row < rows; row++) {
    for (col = 0; col < cols; col++)
      m[row][col] = v;
  }
}

INLINE void
mat_uchar_set2(
unsigned char **m,
const int     rows,
const int     cols,
const int     v
)
{
  int   row, col;

  for (row = 0; row < rows; row++) {
    for (col = 0; col < cols; col++)
      m[row][col] = v;
  }
}

INLINE void
mat_double_set2(
double **m,
const int     rows,
const int     cols,
const double  v
)
{
  int   row, col;

  for (row = 0; row < rows; row++) {
    for (col = 0; col < cols; col++)
      m[row][col] = v;
  }
}

INLINE void
mat_set(
int **m,
const int     n,
const int     v
)
{
  mat_set2(m,n,n,v);
}

INLINE void
mat_uchar_set(
unsigned char **m,
const int     n,
const int     v
)
{
  mat_uchar_set2(m,n,n,v);
}

/*  Leave first a rows and first a columns alone; fill rest of matrix
    with 0's.
*/
INLINE void
mat_uchar_zero_lowright(
unsigned char   * const * const m,
const int       n,
const int       a
)
{
  int   row,col;

  for (row = a; row < n; row++) {
    for (col = a; col < n; col++)
      m[row][col] = 0;
  }
}

#ifdef  OLD_DIFFERENT_FUNCTION
void
mat_uchar_sum2(
unsigned char **m,
const int n,
const int a,
int v[]
)
{
  int   row, col, limit;

  for (row = 0; row < n; row++) {
    v[row] = 0;
    limit = (row < a) ? n-1 : a-1;
    for (col = 0; col <= limit; col++)
      v[row] += m[row][col];
  }
}

/* find sum of each row of a matrix */
void
mat_uchar_sum(
unsigned char   **m,  /* matrix */
const int       n,              /* matrix is n*n */
int             v[]             /* where to put sum */
)
{
  int   row, col;

  /* whole thing is equivalent to mat_uchar_sum2(m,n,n,v) */
  for (row = 0; row < n; row++) {
    v[row] = 0;
    for (col = 0; col < n; col++) 
      v[row] += m[row][col];
  }
}
#endif  /*OLD_DIFFERENT_FUNCTION*/

INLINE void
mat_add_scalar2(
int   **m,
const int rows,
const int cols,
const int s
)
{
  int   i,j;

  for (i = 0; i < rows; i++) {
    for (j = 0; j < cols; j++) {
      m[i][j] += s;
    }
  }
}

INLINE void
mat_add_scalar(
int   **m,
const int n,
const int s
)
{
  mat_add_scalar2(m,n,n,s);
}

INLINE void
mat_uchar_add_scalar2(
unsigned char   **m,
const int rows,
const int cols,
const int s
)
{
  int   i,j;

  for (i = 0; i < rows; i++) {
    for (j = 0; j < cols; j++) {
      m[i][j] += s;
    }
  }
}

INLINE void
mat_uchar_add_scalar(
unsigned char   **m,
const int n,
const int s
)
{
  mat_uchar_add_scalar2(m,n,n,s);
}

INLINE int
mat_sum2(
int         **m,
const int   rows,
const int   cols
)
{
  int   row,col,v;
  int   sum;

  sum = 0;
  for (row = 0; row < rows; row++) {
    for (col = 0; col < cols; col++) {
      v = m[row][col];
      sum += v;
    }
  }
  return(sum);
}


/*  Sum the individual elements of an INT matrix.  Returns -1 if any elements
    are negative; else returns the sum.
*/
INLINE int
mat_positive_sum2(
int         **m,
const int   rows,
const int   cols
)
{
  int   row,col,v;
  int   sum;

  sum = 0;
  for (row = 0; row < rows; row++) {
    for (col = 0; col < cols; col++) {
      v = m[row][col];
      if (v < 0)
        return(-1);
      sum += v;
    }
  }
  return(sum);
}

/* Compute a vector vec[i], where vec[i] is the sum of the row i elements of m,
   i=0..rows-1.
   Assumes overflow is not going to occur.
*/
INLINE void
mat_uchar_row_sum2(
unsigned char * const * const m,
const int rows,
const int cols,
int * const sums_ret
)
{
  int row,col,s;

  for (row = 0; row < rows; row++) {
    const unsigned char * const rowvec = m[row];
    for (s = 0, col = 0; col < cols; col++)  s += rowvec[col];
    sums_ret[row] = s;
  }
}

INLINE void
mat_uchar_row_sum(
unsigned char * const * const m,
const int n,
int * const sums_ret
)
{
  mat_uchar_row_sum2(m,n,n,sums_ret);
}

/* Just like mat_uchar_row_sum, but allocates space for the row sum vector.
   This function can be used to compute the degree of each vertex in
   a graph, e.g.,

       int * degrees = mat_uchar_alloc_row_sum(adjacency,n,xmalloc);
*/
INLINE int *
mat_uchar_alloc_row_sum(
unsigned char * const * const m,
const int n,
void  *(*allocfunc) P((const int))
)
{
  /* We could use vec_alloc, but that would mean we would have to include
     vector.h, and it is nice to avoid that.
  */
  int * const sums_ret = (*allocfunc) (n * sizeof(sums_ret[0]));

  mat_uchar_row_sum2(m,n,n,sums_ret);
  return(sums_ret);
}

/* Compute a vector vec[i], where vec[i] is the sum of the row i elements of m,
   i=0..rows-1.
   Assumes overflow is not going to occur.
*/
INLINE void
mat_row_sum2(
int * const * const m,
const int rows,
const int cols,
int * const sums_ret
)
{
  int row,col,s;

  for (row = 0; row < rows; row++) {
    const int * const rowvec = m[row];
    for (s = 0, col = 0; col < cols; col++)  s += rowvec[col];
    sums_ret[row] = s;
  }
}

INLINE void
mat_row_sum(
int * const * const m,
const int n,
int * const sums_ret
)
{
  mat_row_sum2(m,n,n,sums_ret);
}


INLINE int
mat_uchar_sum2(
unsigned char     **m,
const int         rows,
const int         cols
)
{
  int   row,col;
  int   sum;

  sum = 0;
  for (row = 0; row < rows; row++) {
    for (col = 0; col < cols; col++)
      sum += m[row][col];
  }
  return(sum);
}

INLINE double
mat_double_sum2(
double            **m,
const int         rows,
const int         cols
)
{
  int     row,col;
  double  sum;

  sum = 0.0;
  for (row = 0; row < rows; row++) {
    for (col = 0; col < cols; col++)
      sum += m[row][col];
  }
  return(sum);
}

INLINE int
mat_positive_sum(
int         **m,
const int   n
)
{
  return(mat_positive_sum2(m,n,n));
}

INLINE int
mat_uchar_sum(
unsigned char     **m,
const int         n
)
{
  return(mat_uchar_sum2(m,n,n));
}

/*  Sum the squares of the individual elements of an int matrix.
*/
INLINE int
mat_sum_squares2(
int         **m,
const int   rows,
const int   cols
)
{
  int   row,col,v;
  int   sum;

  sum = 0;
  for (row = 0; row < rows; row++) {
    for (col = 0; col < cols; col++) {
      v = m[row][col];
      sum += v*v;
    }
  }
  return(sum);
}

INLINE int
mat_uchar_sum_squares2(
unsigned char   **m,
const int       rows,
const int       cols
)
{
  int   row,col,v;
  int   sum;

  sum = 0;
  for (row = 0; row < rows; row++) {
    for (col = 0; col < cols; col++) {
      v = m[row][col];
      sum += v*v;
    }
  }
  return(sum);
}

INLINE int
mat_sum_squares(
int             **m,
const int       n
)
{
  return(mat_sum_squares2(m, n, n));
}

INLINE int
mat_uchar_sum_squares(
unsigned char **m,
const int       n
)
{
  return(mat_uchar_sum_squares2(m, n, n));
}

/*  Take the transpose of an n*n matrix.
*/
INLINE void
mat_transpose(
int         **dest,
int         **source,   /*  const int **  */
const int   n
)
{
  int   i, j;
  int   *dest_vec;

  for (i = 0; i < n; i++) {
    dest_vec = dest[i];
    for (j = 0; j < n; j++) {
      dest_vec[j] = source[j][i];
    }
  }
}

INLINE void
mat_uchar_minmax2(
unsigned char     * const * const m,
const int         rows,
const int         cols,
int               * const minret,
int               * const maxret
)
{
  int   row,col,min,max;

  min = max = m[0][0];
  for (row = 0; row < rows; row++) {
    for (col = 0; col < cols; col++) {
      max = MAX(max, m[row][col]);
      min = MIN(min, m[row][col]);
    }
  }
  if (minret) *minret = min;
  if (maxret) *maxret = max;
}

INLINE void
mat_double_minmax2(
double        * const * const m,
const int     rows,
const int     cols,
double        * const minret,
double        * const maxret
)
{
  int   row,col;
  double  min,max;

  min = max = m[0][0];
  for (row = 0; row < rows; row++) {
    for (col = 0; col < cols; col++) {
      max = MAX(max, m[row][col]);
      min = MIN(min, m[row][col]);
    }
  }
  if (minret) *minret = min;
  if (maxret) *maxret = max;
}

/*  Find the smallest and largest values in an INT matrix. */
INLINE void
mat_minmax2(
int         * const * const m,
const int   rows,
const int   cols,
int         * const minret,
int         * const maxret
)
{
  int   row,col,min,max;

  if ((rows > 0) && (cols > 0)) {
    min = max = m[0][0];
    for (row = 0; row < rows; row++) {
      for (col = 0; col < cols; col++) {
        max = MAX(max, m[row][col]);
        min = MIN(min, m[row][col]);
      }
    }
    if (minret) *minret = min;
    if (maxret) *maxret = max;
  }
}

INLINE void
mat_minmax(
int         * const * const m,
const int   n,
int         * const minret,
int         * const maxret
)
{
  mat_minmax2(m,n,n,minret,maxret);
}

INLINE int
mat_uchar_cmp2(
unsigned char **m1,
unsigned char **m2,
const int n,
const int a
)
{
  int   row, col, limit;

  for (row = 0; row < n; row++) {
    limit = (row < a) ? n-1 : a-1;
    for (col = 0; col <= limit; col++) {
      if (m1[row][col] != m2[row][col])
        return(m1[row][col] - m2[row][col]);
    }
  }
  return(0);
}

INLINE int
mat_cmp2(
int **m1,
int **m2,
const int n,
const int a
)
{
  int   row, col, limit;

  for (row = 0; row < n; row++) {
    limit = (row < a) ? n-1 : a-1;
    for (col = 0; col <= limit; col++) {
      if (m1[row][col] != m2[row][col])
        return(m1[row][col] - m2[row][col]);
    }
  }
  return(0);
}

INLINE int
mat_uchar_cmp(
unsigned char **m1,
unsigned char **m2,
const int n
)
{
  return(mat_uchar_cmp2(m1,m2,n,n));
}

INLINE int
mat_cmp(
int **m1,
int **m2,
const int n
)
{
  return(mat_cmp2(m1,m2,n,n));
}

INLINE int
mat_is0(
int             **m,
const int       n
)
{
  int   row,col;

  for (row = 0; row < n; row++) {
    for (col = 0; col < n; col++) {
      if (m[row][col])
        return(0);
    }
  }
  return(1);
}

INLINE int
mat_uchar_is_symmetric2(
unsigned char   **m,
const int       n,
const int       a
)
{
  int   row,col;

  for (row = 0; row < a; row++) {
    for (col = row+1; col < n; col++) {
      if (m[row][col] != m[col][row])
        return(0);
    }
  }
  return(1);
}

INLINE int
mat_is_symmetric2(
int         **m,
const int   n,
const int   a
)
{
  int   row,col;

  for (row = 0; row < a; row++) {
    for (col = row+1; col < n; col++) {
      if (m[row][col] != m[col][row])
        return(0);
    }
  }
  return(1);
}

INLINE int
mat_uchar_is_symmetric(
unsigned char   **m,
const int       n
)
{
  return(mat_uchar_is_symmetric2(m,n,n));
}

INLINE int
mat_is_symmetric(
int       **m,
const int n
)
{
  return(mat_is_symmetric2(m,n,n));
}

INLINE void
mat_zero_3d(
int         ** const * const m,
const int   d1,
const int   d2,
const int   d3
)
{
  int   a,b,c;

  for (a = 0; a < d1; a++) {
    for (b = 0; b < d2; b++) {
      for (c = 0; c < d3; c++)
        m[a][b][c] = 0;
    }
  }
}

/*  Apply a mapping of values to a matrix.
    It is allowable to have source == dest.
*/
INLINE void
mat_apply_map(
int         **dest,
const int   rows,
const int   cols,
const int   min,  /* min. value occuring in source */
const int   *map, /* map[0] == what to map min to; map[1] map for min+1, etc. */
int         **source            /* const int ** */
)
{
  int   row, col;

  for (row = 0; row < rows; row++) {
    for (col = 0; col < cols; col++) {
      dest[row][col] = map[source[row][col]-min];
    }
  }
}

INLINE void
mat_uchar_apply_map(
unsigned char       **dest,
const int           rows,
const int           cols,
const int           min,
const int           *map,
unsigned char       **source    /* const unsigned char ** */
)
{
  int   row, col;

  for (row = 0; row < rows; row++) {
    for (col = 0; col < cols; col++) {
      dest[row][col] = map[source[row][col]-min];
    }
  }
}

#endif  /* defined(__GNUC__) || defined(source file) */

#endif /* _020992_124132_matrix */
